#include "IFB.h"

SamplingLocation::SamplingLocation(V3 pP, int _u0, int _v0) {

	uv1 = pP;
	uv1[2] = 1.0f;
	z = pP[2];
	u0 = _u0;
	v0 = _v0;

}

SuperPixel::SuperPixel() {

	slsAN = 0;
	slsN = 0;
	sls = 0;

}

IFB::IFB(int _w, int _h) {

	w = _w;
	h = _h;
	spix = new SuperPixel[w*h];

}


void IFB::AddSamplingLocation(V3 pP, int u0, int v0) {
	float uf = pP[0];
	float vf = pP[1];

	if (uf < 0.0f || uf >= (float)w)
		return;
	if (vf < 0.0f || vf >= (float)h)
		return;
	spix[(h - 1 - (int)vf)*w + (int)uf].AddSamplingLocation(pP, u0, v0);

}


void SuperPixel::AddSamplingLocation(V3 pP, int u0, int v0) {

	if (slsAN == slsN) {
		slsAN *= 2;
		if (slsAN == 0)
			slsAN = 1;
		SamplingLocation *newsls = new SamplingLocation[slsAN];
		for (int sli = 0; sli < slsN; sli++) {
			newsls[sli] = sls[sli];
		}
		if (sls)
			delete []sls;
		sls = newsls;
	}

	sls[slsN] = SamplingLocation(pP, u0, v0);
	slsN++;

}


int IFB::Min() {

	int ret = 10000;
	for (int v = 0; v < h; v++) {
		for (int u = 0; u < w; u++) {
			if (spix[(h - 1 - v)*w + u].slsN < ret)
				ret = spix[(h - 1 - v)*w + u].slsN;
		}
	}
	return ret;

}

int IFB::Max() {

	int ret = -1;
	for (int v = 0; v < h; v++) {
		for (int u = 0; u < w; u++) {
			if (spix[(h - 1 - v)*w + u].slsN > ret)
				ret = spix[(h - 1 - v)*w + u].slsN;
		}
	}
	return ret;

}

void IFB::Visualize(FrameBuffer *visfb, V3 c0, V3 c1, int minsls, int maxsls) {

	if (maxsls == -1) {
		minsls = Min();
		maxsls = Max();
	}
	visfb->SetBGR(0xFF000000);
	for (int v = 0; v < h; v++) {
		for (int u = 0; u < w; u++) {
			float fracf = (spix[(h - 1 - v)*w + u].slsN - (float) minsls) /
				((float)maxsls - (float)minsls);
			if (fracf > 1.0f) fracf = 1.0f;
			if (fracf < 0.0f) fracf = 0.0f;
			V3 c = c0 + (c1 - c0)*fracf;
			visfb->Set(u, v, c.GetColor());
		}
	}

}

SuperPixel* IFB::Get(int u, int v) {

	if (u < 0 || u > w - 1 || v < 0 || v > h - 1)
		return 0;
	return &(spix[(h - 1 - v)*w + u]);

}



void IFB::AddSamplingLocations(PPC *ifbppc, FrameBuffer *fb, PPC *ppc) {

	for (int v = 0; v < fb->h; v++) {
		for (int u = 0; u < fb->w; u++) {
			float currz = fb->GetZ(u, v);
			if (currz == 0.0f)
				continue;
			V3 P;
			ppc->UnProject(V3(.5f + (float)u, .5f + (float)v, currz), P);
			V3 Q;
			if (!ifbppc->Project(P, Q))
				continue;
			AddSamplingLocation(Q, u, v);
		}
	}
}

void IFB::Clear() {

	if (!spix)
		return;

	for (int v = 0; v < h; v++) {
		for (int u = 0; u < w; u++) {
			SuperPixel &sp = *Get(u, v);
			if (sp.slsAN) {
				delete[]sp.sls;
				sp.sls = 0;
				sp.slsAN = sp.slsN = 0;
			}
		}
	}

}